package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdEcuBrandLabelBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.entity.request.TagConsumeRequest;
import com.gic.spark.util.CommonUtil;
import org.apache.spark.api.java.JavaRDD;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * 连带率
 * @author: wangxk
 * @date: 2020/5/7
 */
public class TagAssociatedPurchaseRateFilter extends AbstractTagConsumFilter{

    private static TagAssociatedPurchaseRateFilter instance;
    public static TagAssociatedPurchaseRateFilter getInstance() {
        if(null==instance){
            instance=new TagAssociatedPurchaseRateFilter();
        }
        return instance;
    }
    private TagAssociatedPurchaseRateFilter(){}
    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHive);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeDoubleRequest consumeRequest=(TagConsumeDoubleRequest)request;
        JavaRDD<TrdEcuBrandLabelBean> consumeRDD= MysqlRddManager.getPojoFromDataset(dataSourceHive.getDatasetByEntId(enterpriseId), TrdEcuBrandLabelBean.class).javaRDD();
        consumeRDD=statisticsTypeHandle(consumeRDD,consumeRequest);
        JavaRDD<Long>ecuRdd=consumeRDD.mapToPair(data-> Tuple2.apply(data.getEcu_id(),data))
                                        .reduceByKey((x,y)->{
                                            x.setSeff_goods_num(x.getSeff_goods_num()+y.getSeff_goods_num());
                                            x.setSeff_order_cnt(x.getSeff_order_cnt()+y.getSeff_order_cnt());
                                            return x;
                                        })
                                        .mapPartitions(data->{
                                            List<Long> result=new ArrayList();
                                            while (data.hasNext()){
                                                Tuple2<Long,TrdEcuBrandLabelBean> tp2=data.next();
                                                double jointRate= CommonUtil.isEmptyInteger2int(tp2._2().getSeff_goods_num())/CommonUtil.isEmptyInteger2int(tp2._2().getSeff_order_cnt());
                                                switch (consumeRequest.getNumberType()){
                                                    case gt:
                                                        if(jointRate>consumeRequest.getBeginNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case gte:
                                                        if(jointRate>=consumeRequest.getBeginNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case lt:
                                                        if(jointRate<consumeRequest.getEndNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case lte:
                                                        if(jointRate<=consumeRequest.getEndNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case eq:
                                                        if(jointRate==consumeRequest.getEqualNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case between:
                                                        if(jointRate>=consumeRequest.getBeginNum()
                                                                &&jointRate<=consumeRequest.getEndNum()){
                                                            result.add(tp2._1());
                                                        }
                                                    default:break;
                                                }
                                            }
                                            return result.iterator();
                                        });
        return ecuRdd;
    }
}
