package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdEcuBrandLabelBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.entity.request.TagConsumeRequest;
import com.gic.spark.util.CommonUtil;
import org.apache.spark.api.java.JavaRDD;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * 平均折扣率
 * @author: wangxk
 * @date: 2020/5/7
 */
public class TagAverageDiscountFactorFilter extends AbstractTagConsumFilter {

    private static TagAverageDiscountFactorFilter instance;
    public static TagAverageDiscountFactorFilter getInstance() {
        if(null==instance){
            instance=new TagAverageDiscountFactorFilter();
        }
        return instance;
    }
    private TagAverageDiscountFactorFilter(){}


    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHive);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeDoubleRequest consumeRequest=(TagConsumeDoubleRequest)request;
        JavaRDD<TrdEcuBrandLabelBean> consumeRDD= MysqlRddManager.getPojoFromDataset(dataSourceHive.getDatasetByEntId(enterpriseId), TrdEcuBrandLabelBean.class).javaRDD();
        consumeRDD=statisticsTypeHandle(consumeRDD,consumeRequest);

        int configStatus= CommonUtil.getConfigStatus(enterpriseId);
        JavaRDD<Long>ecuRdd=consumeRDD.mapToPair(data-> Tuple2.apply(data.getEcu_id(),data))
                                      .reduceByKey((x,y)->{
                                            x.setReceive_amt(x.getReceive_amt()+y.getReceive_amt());
                                            x.setPay_amt(x.getPay_amt()+y.getPay_amt());
                                            x.setTotal_amt(x.getTotal_amt()+y.getTotal_amt());
                                            return x;
                                      })
                                        .mapPartitions(data->{
                                           List<Long>result=new ArrayList();
                                           while (data.hasNext()){
                                               Tuple2<Long,TrdEcuBrandLabelBean> tp2=data.next();
                                               double avgDiscountRate=1==configStatus?CommonUtil.isEmptyDouble2double(tp2._2().getPay_amt())/CommonUtil.isEmptyDouble2double(tp2._2().getTotal_amt())
                                                                                      :CommonUtil.isEmptyDouble2double(tp2._2().getReceive_amt())/CommonUtil.isEmptyDouble2double(tp2._2().getTotal_amt());
                                               switch (consumeRequest.getNumberType()){
                                                   case gt:
                                                       if(avgDiscountRate>consumeRequest.getBeginNum()){
                                                           result.add(tp2._1());
                                                       }
                                                       break;
                                                   case gte:
                                                       if(avgDiscountRate>=consumeRequest.getBeginNum()){
                                                           result.add(tp2._1());
                                                       }
                                                       break;
                                                   case lt:
                                                       if(avgDiscountRate<consumeRequest.getEndNum()){
                                                           result.add(tp2._1());
                                                       }
                                                       break;
                                                   case lte:
                                                       if(avgDiscountRate<=consumeRequest.getEndNum()){
                                                           result.add(tp2._1());
                                                       }
                                                       break;
                                                   case eq:
                                                       if(avgDiscountRate==consumeRequest.getEqualNum()){
                                                           result.add(tp2._1());
                                                       }
                                                       break;
                                                   case between:
                                                       if(avgDiscountRate>=consumeRequest.getBeginNum()
                                                               &&avgDiscountRate<=consumeRequest.getEndNum()){
                                                           result.add(tp2._1());
                                                       }
                                                   default:break;
                                               }
                                           }
                                           return result.iterator();
                                        });
        return ecuRdd;
    }
}
