package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdVirtualOrderBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeStoreRequest;
import com.gic.spark.util.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import org.apache.spark.sql.Row;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/8/11
 */
public class TagFirstOfflineConsumptionStoreFilter extends AbstractTagConsumRecordFilter{

    private static TagFirstOfflineConsumptionStoreFilter instance;

    public static TagFirstOfflineConsumptionStoreFilter getInstance() {
        return instance;
    }

    private TagFirstOfflineConsumptionStoreFilter(){}

    public static void setInstance(TagFirstOfflineConsumptionStoreFilter instance) {
        TagFirstOfflineConsumptionStoreFilter.instance = instance;
    }

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveOrder);
        result.add(dataSourceHiveOrderItem);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeStoreRequest storeRequest=(TagConsumeStoreRequest)request;
        JavaRDD<TrdVirtualOrderBean>consumeRecordRDD= MysqlRddManager.getPojoFromDataset(dataSourceHiveOrder.getDatasetByEntId(enterpriseId),TrdVirtualOrderBean.class).javaRDD();

        JavaRDD<Row>virtualOrderItemRdd=dataSourceHiveOrderItem.getDatasetByEntId(enterpriseId).select("virtual_order_id","ent_brand_id").javaRDD();

        JavaRDD<Tuple2<TrdVirtualOrderBean,Optional<Iterable<String>>>>orderRdd=consumeRecordRDD.mapToPair(data->Tuple2.apply(data.getVirtual_id(),data))
                                                                                .leftOuterJoin(virtualOrderItemRdd.mapToPair(row->Tuple2.apply(row.getLong(0),row.getString(1))).groupByKey())
                                                                                .map(data->data._2());

        consumeRecordRDD=statisticsTypeHandle(orderRdd,storeRequest);
        JavaRDD<Long>ecuRdd=consumeRecordRDD.filter(data->data.getOrder_channel_code()==1
                                                &&StringUtils.isNotEmpty(data.getReceipts_time())
                                                &&null!=data.getStore_info_id())
                                                .mapToPair(data-> Tuple2.apply(data.getEcu_id(),Tuple2.apply(data.getReceipts_time(),data.getStore_info_id())))
                                                .reduceByKey((x,y)->{
                                                    if(DateUtil.strToDate(x._1(),DateUtil.FORMAT_DATETIME_19).getTime()
                                                            <DateUtil.strToDate(y._1(),DateUtil.FORMAT_DATETIME_19).getTime()){
                                                        return x;
                                                    }else{
                                                        return y;
                                                    }
                                                }).filter(data->storeRequest.getStoreList().contains(data._2()._2()))
                                                .map(data->data._1());

        return ecuRdd;
    }
}
